'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib_1 = require('tslib');
var React = require('react');
var React__default = _interopDefault(React);
var reactDnd = require('react-dnd');
var reactResizable = require('react-resizable');
var lodash = require('lodash');
var Html5Backend = _interopDefault(require('react-dnd-html5-backend'));
var TouchBackend = _interopDefault(require('react-dnd-touch-backend'));
var MarkDown = _interopDefault(require('react-markdown'));
var reactColor = require('react-color');

function updateNode(byId, newNode) {
  var _a;

  return tslib_1.__assign({}, byId, (_a = {}, _a[newNode.id] = newNode, _a));
}
function removeNode(byId, nodeId) {
  var _a;

  var node = byId[nodeId];
  if (!node) return byId;

  if (node && node.parentId) {
    var parent_1 = byId[node.parentId];

    if (parent_1) {
      var childIdx = parent_1.childrenIds.findIndex(function (id) {
        return id === node.id;
      });

      if (childIdx >= 0) {
        var childrenIds = parent_1.childrenIds.filter(function (id) {
          return id !== node.id;
        });
        byId = updateNode(byId, tslib_1.__assign({}, parent_1, {
          childrenIds: childrenIds
        }));
      }
    }
  }

  return tslib_1.__assign({}, byId, (_a = {}, _a[nodeId] = undefined, _a));
}
function hasDescendent(byId, nodeId, descendentId, checked) {
  if (checked === void 0) {
    checked = {};
  }

  if (checked[nodeId]) return true; //CYCLE

  checked[nodeId] = true;
  var node = byId[nodeId];

  if (!node) {
    return false; //node not found? bad case
  }

  if (node.childrenIds.indexOf(descendentId) >= 0) {
    return true;
  }

  for (var _i = 0, _a = node.childrenIds; _i < _a.length; _i++) {
    var childId = _a[_i];
    var descendentOfChild = hasDescendent(byId, childId, descendentId, checked);
    if (descendentOfChild) return true;
  }

  return false;
}
function placeNodeInParent(byId, node, newParentId, opts) {
  if (!node) {
    return byId;
  } // Prevent placing a parent under a current child (or descendent) to avoid cycles


  if (hasDescendent(byId, node.id, newParentId)) {
    console.warn('node ', node.id, ' has ', newParentId, ' as child, so cannot make it into', node.id, "'s parent OR there's a cycle");
    return byId;
  } // Disallow placing within itself


  if (node.id === newParentId) {
    console.warn('Cannot place into self: ', {
      id: node.id,
      newParentId: newParentId
    });
    return byId;
  } // if parent changed, remove node from old parent


  if (node && node.parentId && node.parentId !== newParentId) {
    var parent_2 = byId[node.parentId];

    if (parent_2) {
      var childrenIds = parent_2.childrenIds.filter(function (id) {
        return id !== node.id;
      });
      byId = updateNode(byId, tslib_1.__assign({}, parent_2, {
        childrenIds: childrenIds
      }));
    }
  }

  var newParent = byId[newParentId];

  if (newParent) {
    var childrenIds = reinsertIntoList(newParent.childrenIds, node.id, opts);
    byId = updateNode(byId, tslib_1.__assign({}, newParent, {
      childrenIds: childrenIds
    }));
  }

  return updateNode(byId, tslib_1.__assign({}, node, {
    parentId: newParentId
  }));
}
function reinsertIntoList(ids, itemId, opts) {
  if (opts === void 0) {
    opts = {};
  } // Disallow placing before/after itself


  if (itemId === opts.beforeItemId || itemId === opts.afterItemId) {
    console.warn('Cannot place relative to self: ', tslib_1.__assign({
      itemId: itemId
    }, opts));
    return ids;
  } // remove the item if it's in the list


  ids = ids.filter(function (id) {
    return id !== itemId;
  });
  var beforeItemId = opts.beforeItemId,
      afterItemId = opts.afterItemId;
  var refItemIdx = -1;

  if (beforeItemId) {
    refItemIdx = ids.findIndex(function (id) {
      return id === beforeItemId;
    });
  } else if (afterItemId) {
    refItemIdx = ids.findIndex(function (id) {
      return id === afterItemId;
    }) + 1;
  }

  if (refItemIdx >= 0) {
    var newItems = ids.slice();
    newItems.splice(refItemIdx, 0, itemId);
    return newItems;
  }

  return ids.concat([itemId]);
}

var cloneDeep =
/*#__PURE__*/
require('lodash').cloneDeep;

function deepCopy(byId, node) {
  if (!node) return node;
  return tslib_1.__assign({}, node, {
    value: cloneDeep(node.value),
    childrenIds: node.childrenIds.slice(),
    children: node.childrenIds.map(function (id) {
      var childNode = byId[id];
      return deepCopy(byId, childNode);
    })
  });
}
function paste(value) {
  var byId = value.byId;
  var copiedNode = (value ? value : {
    copiedNode: null
  }).copiedNode;
  if (!copiedNode) return value; // support pasting same node multiple times - ensure childrenIds in each copy point to different arrays

  copiedNode = cloneDeep(copiedNode);
  var newById = pasteChild(byId, copiedNode).byId; // add copied node to parent

  if (copiedNode.parentId) {
    var parentNode = byId[copiedNode.parentId];

    if (parentNode) {
      return tslib_1.__assign({}, value, {
        byId: placeNodeInParent(newById, copiedNode, copiedNode.parentId, {
          afterItemId: parentNode.childrenIds[parentNode.childrenIds.length - 1]
        })
      });
    }
  }

  return value;
}
function pasteChild(byId, copiedNode) {
  var _a;

  copiedNode.id = newBlockId(byId, copiedNode.type);
  var children = copiedNode.children;
  copiedNode.children = undefined;
  byId = tslib_1.__assign({}, byId, (_a = {}, _a[copiedNode.id] = copiedNode, _a));

  if (children) {
    var idx = 0;

    for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
      var child = children_1[_i];
      child.parentId = copiedNode.id;

      var _b = pasteChild(byId, child),
          newById = _b.byId,
          newChildId = _b.newId;

      copiedNode.childrenIds[idx] = newChildId;
      byId = newById;
      ++idx;
    }
  }

  return {
    byId: byId,
    newId: copiedNode.id
  };
}
function newBlockId(byId, prefix) {
  if (prefix === void 0) {
    prefix = 'node';
  } //TODO better way to create uid


  var nextId = Object.keys(byId).length;

  while (!!byId[nextId]) nextId++;

  return prefix + '_' + nextId;
}
function create(value, props) {
  var byId = value.byId;
  var nodeId = newBlockId(byId);

  var node = tslib_1.__assign({}, byId[nodeId], props);

  if (!node) return value;
  return tslib_1.__assign({}, value, {
    byId: updateNode(byId, node)
  });
}
function update(value, nodeId, propsToUpdate) {
  var byId = value.byId;

  var node = tslib_1.__assign({}, byId[nodeId], propsToUpdate);

  if (!node) return value;
  return tslib_1.__assign({}, value, {
    byId: updateNode(byId, node)
  });
}
function destroy(value, nodeId) {
  var byId = value.byId,
      rootNodeId = value.rootNodeId,
      focusedNodeId = value.focusedNodeId;

  if (rootNodeId === nodeId) {
    alert('Cannot destroy root node');
    return tslib_1.__assign({}, value, {
      byId: byId,
      focusedNodeId: focusedNodeId
    });
  }

  return tslib_1.__assign({}, value, {
    byId: removeNode(byId, nodeId),
    focusedNodeId: focusedNodeId === nodeId ? null : focusedNodeId
  });
}
function move(value, nodeId, targetParentId, opts) {
  if (opts === void 0) {
    opts = {};
  }

  if (!targetParentId) return value;
  var byId = value.byId;
  var node = byId[nodeId];
  return tslib_1.__assign({}, value, {
    byId: placeNodeInParent(byId, tslib_1.__assign({}, node, {
      isPlaceHolder: opts.isPlaceHolder
    }, opts.absolutePos ? {
      top: opts.absolutePos.top,
      left: opts.absolutePos.left
    } : {}), targetParentId, opts)
  });
}
function moveInDirection(value, nodeId, moveOpts) {
  if (moveOpts === void 0) {
    moveOpts = {};
  }

  var byId = value.byId;
  var node = byId[nodeId];
  if (!node) return value;
  var targetParentId = node.parentId;
  if (!targetParentId) return value; // sanity check

  var parentNode = byId[targetParentId];
  if (!parentNode || !parentNode.parentId) return value; // sanity check

  var grandParentNode = byId[parentNode.parentId]; // if (!parentNode) return value; // sanity check

  var curPositionInParent = parentNode.childrenIds.indexOf(nodeId);
  if (curPositionInParent < 0) return value; // sanity check

  var opts = {};

  if (parentNode.type === 'row') {
    if (moveOpts.direction === 'left') {
      if (curPositionInParent === 0) return value;
      opts = {
        beforeItemId: parentNode.childrenIds[curPositionInParent - 1]
      };
    } else if (moveOpts.direction === 'right') {
      if (curPositionInParent >= parentNode.childrenIds.length - 1) return value;
      opts = {
        afterItemId: parentNode.childrenIds[curPositionInParent + 1]
      };
    } else {
      if (grandParentNode && grandParentNode.type === 'col') {
        var parentPositionInGrandparent = grandParentNode.childrenIds.indexOf(parentNode.id);

        if (parentPositionInGrandparent > 0 && moveOpts.direction === 'up') {
          // move to uncle (grandparent's other child)
          return move(value, nodeId, grandParentNode.childrenIds[parentPositionInGrandparent - 1]);
        } else if (parentPositionInGrandparent < grandParentNode.childrenIds.length - 1 && moveOpts.direction === 'down') {
          return move(value, nodeId, grandParentNode.childrenIds[parentPositionInGrandparent + 1]);
        } else {
          return value;
        }
      } else {
        return value;
      }
    }
  } else if (parentNode.type === 'col') {
    if (moveOpts.direction === 'up') {
      if (curPositionInParent === 0) return value;
      opts = {
        beforeItemId: parentNode.childrenIds[curPositionInParent - 1]
      };
    } else if (moveOpts.direction === 'down') {
      if (curPositionInParent >= parentNode.childrenIds.length - 1) return value;
      opts = {
        afterItemId: parentNode.childrenIds[curPositionInParent + 1]
      };
    } else {
      if (grandParentNode && grandParentNode.type === 'row') {
        var parentPositionInGrandparent = grandParentNode.childrenIds.indexOf(parentNode.id);

        if (parentPositionInGrandparent > 0 && moveOpts.direction === 'left') {
          return move(value, nodeId, grandParentNode.childrenIds[parentPositionInGrandparent - 1]);
        } else if (parentPositionInGrandparent < grandParentNode.childrenIds.length - 1 && moveOpts.direction === 'right') {
          // move to uncle (grandparent's other child)
          return move(value, nodeId, grandParentNode.childrenIds[parentPositionInGrandparent + 1]);
        } else {
          return value;
        }
      } else {
        return value;
      }
    }
  } else if (parentNode.type === 'layer') {
    // move by stepPx if inside layer
    var topDelta = (moveOpts.direction === 'up' ? -1 : moveOpts.direction === 'down' ? 1 : 0) * (moveOpts.stepPx || 1);
    var leftDelta = (moveOpts.direction === 'left' ? -1 : moveOpts.direction === 'right' ? 1 : 0) * (moveOpts.stepPx || 1);
    opts = {
      absolutePos: {
        top: (node.top || 0) + topDelta,
        left: (node.left || 0) + leftDelta
      }
    };
  }

  return tslib_1.__assign({}, value, {
    byId: placeNodeInParent(byId, tslib_1.__assign({}, node, {
      isPlaceHolder: opts.isPlaceHolder
    }, opts.absolutePos ? {
      top: opts.absolutePos.top,
      left: opts.absolutePos.left
    } : {}), targetParentId, opts)
  });
}
function focusNode(value, node, focus) {
  if (focus === void 0) {
    focus = true;
  }

  if (!node) return value;
  var focusedNodeId = value.focusedNodeId;
  return tslib_1.__assign({}, value, {
    focusedNodeId: focus ? node.id : focusedNodeId === node.id ? null : focusedNodeId
  });
}

function parseTypes(types) {
  var draggedNodeIds = types.filter(function (t) {
    return t.indexOf('draggednodeid:') === 0;
  });
  var draggedNodeTypes = types.filter(function (t) {
    return t.indexOf('draggednodetype:') === 0;
  });

  if (draggedNodeIds.length !== 1 || draggedNodeTypes.length !== 1) {
    return {
      draggedNodeId: 'notanode',
      draggedNodeType: null
    };
  }

  var draggedNodeId = draggedNodeIds[0].slice('draggednodeid:'.length);
  var draggedNodeType = draggedNodeTypes[0].slice('draggednodetype:'.length);
  return {
    draggedNodeId: draggedNodeId,
    draggedNodeType: draggedNodeType
  };
}
function getDragPositionRelativeToTarget(e, targetRect) {
  if (!targetRect) return null;
  return {
    left: e.clientX - targetRect.left,
    top: e.clientY - targetRect.top,
    width: targetRect.width,
    height: targetRect.height
  };
}
function getDragPositionRelativeToTarget2(_a, targetRect) {
  var clientX = _a.clientX,
      clientY = _a.clientY;
  if (!targetRect) return null;
  return {
    left: clientX - targetRect.left,
    top: clientY - targetRect.top,
    width: targetRect.width,
    height: targetRect.height
  };
}
function onDragStart(e, draggedNode, getBoundingRect) {
  e.stopPropagation();
  var relativePos = getDragPositionRelativeToTarget(e, getBoundingRect());

  if (relativePos) {
    e.dataTransfer.setData('text/plain', JSON.stringify({
      startLeft: relativePos.left,
      startTop: relativePos.top
    }));
  }

  e.dataTransfer.setData("draggednodeid:" + draggedNode.id, '');
  e.dataTransfer.setData("draggednodetype:" + draggedNode.type, '');
}
function onDropped(types, targetNodeId, opts) {
  if (opts === void 0) {
    opts = {};
  }

  var draggedNodeId = parseTypes(types).draggedNodeId;
  return function (value) {
    return move(value, draggedNodeId, targetNodeId, opts);
  };
}
function onDropped2(draggedNodeId, targetNodeId, opts) {
  if (opts === void 0) {
    opts = {};
  }

  return function (value) {
    return move(value, draggedNodeId, targetNodeId, opts);
  };
}

var LayerBlock =
/*#__PURE__*/

/** @class */
function (_super) {
  tslib_1.__extends(LayerBlock, _super);

  function LayerBlock() {
    return _super !== null && _super.apply(this, arguments) || this;
  }

  LayerBlock.prototype.renderChild = function (nodeId) {
    var _a = this.props,
        changeBlocks = _a.changeBlocks,
        getNode = _a.getNode,
        undoRedoVersion = _a.undoRedoVersion,
        focusedNodeId = _a.focusedNodeId,
        renderEditBlock = _a.renderEditBlock;
    var node = getNode(nodeId);
    if (!node) return null;
    return renderEditBlock({
      node: node,
      renderEditBlock: renderEditBlock,
      undoRedoVersion: undoRedoVersion,
      changeBlocks: changeBlocks,
      getNode: getNode,
      focusedNodeId: focusedNodeId
    });
  };

  LayerBlock.prototype.render = function () {
    var _this = this;

    var _a = this.props,
        node = _a.node,
        getNode = _a.getNode;
    var childrenIds = node.childrenIds;
    var paddingLeft = (node.paddingLeftPercentWidth !== undefined ? node.paddingLeftPercentWidth : 0.05) * node.width;
    var paddingTop = (node.paddingTopPercentHeight !== undefined ? node.paddingTopPercentHeight : 0.05) * node.height;
    return React.createElement("div", {
      style: {
        position: 'relative',
        opacity: this.props.isDragging ? 0.5 : 1,
        width: node.width,
        height: node.height,
        backgroundColor: node.backgroundColor || '#f5f5f5a3',
        // padding: 5,
        borderRadius: 3
      }
    }, childrenIds.map(function (childId) {
      var node = getNode(childId);
      if (!node) return null;
      var res = React.createElement("div", {
        // className="drag-node"
        key: 'node_' + childId,
        style: {
          position: 'absolute',
          width: node.width,
          height: node.height,
          top: paddingTop,
          left: paddingLeft,
          transform: "translate(" + node.left + "px," + node.top + "px)"
        }
      }, _this.renderChild(childId));
      return res;
    }));
  };

  return LayerBlock;
}(React.Component);

var AbsoluteLayerBlock = function (props) {
  var selfRef = React.useRef(null);

  var _a = reactDnd.useDrag({
    item: {
      type: 'col',
      id: props.node.id
    },
    collect: function (monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    }
  }),
      isDragging = _a[0].isDragging,
      drag = _a[1];

  var _b = reactDnd.useDrop({
    accept: ['col', 'row', 'layer', 'custom', 'image', 'markdown'],
    canDrop: function (item) {
      var draggedNodeType = item.type;
      return draggedNodeType === 'row' || draggedNodeType === 'col' || draggedNodeType === 'markdown' || draggedNodeType === 'image' || draggedNodeType === 'layer' || draggedNodeType === 'custom';
    },
    drop: function (item, monitor) {
      var isOver = monitor.isOver({
        shallow: true
      });
      var clientOffset = monitor.getClientOffset();
      var sourceClientOffset = monitor.getSourceClientOffset();
      var initialClientOffset = monitor.getInitialClientOffset();
      var initialSourceClientOffset = monitor.getInitialSourceClientOffset(); // ?TODO check if already dropped in nested target... instead of shallow isOver check?

      if (!isOver || !item || !sourceClientOffset || !clientOffset || !initialClientOffset || !initialSourceClientOffset) return null; // geometry: figure out whether the dragged element should go after us or before us

      var relativeDraggedPosition = getDragPositionRelativeToTarget2({
        clientX: clientOffset.x,
        clientY: clientOffset.y
      }, getBoundingRect());
      var lastChildId = props.node.childrenIds.length ? props.node.childrenIds[props.node.childrenIds.length - 1] : undefined;
      props.changeBlocks(onDropped2(item.id, props.node.id, relativeDraggedPosition ? {
        afterItemId: lastChildId,
        absolutePos: {
          top: relativeDraggedPosition.top - (initialClientOffset.y - initialSourceClientOffset.y),
          left: relativeDraggedPosition.left - (initialClientOffset.x - initialSourceClientOffset.x)
        }
      } : undefined));
      return {
        type: 'layer',
        id: props.node.id
      };
    },
    collect: function (monitor) {
      var dragSourceItem = monitor.getItem();
      return {
        isOver: monitor.isOver({
          shallow: true
        }),
        initialClientOffset: monitor.getInitialClientOffset(),
        clientOffset: monitor.getClientOffset(),
        dragSourceItemType: dragSourceItem ? dragSourceItem.type : '',
        dragSourceItemId: dragSourceItem ? dragSourceItem.id : '',
        sourceClientOffset: monitor.getSourceClientOffset(),
        initialSourceClientOffset: monitor.getInitialSourceClientOffset()
      };
    }
  }),
      _c = _b[0],
      drop = _b[1];

  var getBoundingRect = function () {
    return selfRef.current && selfRef.current.getBoundingClientRect ? selfRef.current.getBoundingClientRect() : null;
  };

  return React.createElement("div", {
    ref: selfRef,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React.createElement("div", {
    ref: drop,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React.createElement("div", {
    ref: drag,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React.createElement(LayerBlock, tslib_1.__assign({}, props, {
    isDragging: isDragging
  })))));
};

var ColBlock = function (props) {
  var wantToPlaceNext = props.wantToPlaceNext;

  var renderChild = function (nodeId) {
    var getNode = props.getNode,
        changeBlocks = props.changeBlocks,
        undoRedoVersion = props.undoRedoVersion,
        focusedNodeId = props.focusedNodeId,
        renderEditBlock = props.renderEditBlock;
    var node = getNode(nodeId);
    if (!node) return null;
    return renderEditBlock({
      node: node,
      undoRedoVersion: undoRedoVersion,
      renderEditBlock: renderEditBlock,
      focusedNodeId: focusedNodeId,
      changeBlocks: changeBlocks,
      getNode: getNode
    });
  };

  var reactChildren = props.children;
  var node = props.node,
      getNode = props.getNode;
  var childrenIds = node.childrenIds;
  var firstChildPlaceholderHeight = 3;
  var paddingLeft = (node.paddingLeftPercentWidth !== undefined ? node.paddingLeftPercentWidth : 0.05) * node.width;
  var paddingTop = (node.paddingTopPercentHeight !== undefined ? node.paddingTopPercentHeight : 0.05) * node.height;
  var runningHeight = wantToPlaceNext === 'firstChild' ? firstChildPlaceholderHeight : 0;
  return React__default.createElement("div", {
    key: 'col_' + node.id,
    style: {
      display: 'flex',
      opacity: props.isDragging ? 0.5 : 1,
      flexDirection: 'row',
      width: '100%',
      height: '100%'
    }
  }, wantToPlaceNext === 'left' && React__default.createElement("div", {
    style: {
      height: '100%',
      width: 3,
      backgroundColor: 'green'
    }
  }), React__default.createElement("div", {
    style: {
      width: '100%',
      height: '100%',
      position: 'relative',
      borderRadius: 3,
      backgroundColor: props.node.isPlaceHolder ? 'grey' : props.node.backgroundColor || '#ffffffa8',
      borderStyle: 'dashed',
      borderWidth: 1
    }
  }, wantToPlaceNext === 'firstChild' && React__default.createElement("div", {
    style: {
      height: 10,
      width: '100%',
      position: 'absolute',
      top: paddingTop,
      left: paddingLeft,
      transform: "translate(0," + runningHeight + "px)",
      backgroundColor: 'blue'
    }
  }), React__default.Children.count(reactChildren) ? reactChildren : childrenIds.map(function (childId) {
    var row = getNode(childId);
    if (!row) return null;
    var res = React__default.createElement("div", {
      className: "drag-node",
      key: 'node_' + childId,
      style: {
        position: 'absolute',
        width: row.width,
        height: row.height,
        top: paddingTop,
        left: paddingLeft,
        transform: "translate(0," + runningHeight + "px)"
      }
    }, renderChild(childId));
    runningHeight += row.height;
    return res;
  }), wantToPlaceNext === 'lastChild' && React__default.createElement("div", {
    style: {
      top: paddingLeft,
      left: paddingTop,
      transform: "translate(0," + runningHeight + "px)",
      position: 'absolute',
      height: 10,
      width: '100%',
      backgroundColor: 'orange'
    }
  })), wantToPlaceNext === 'right' && React__default.createElement("div", {
    style: {
      height: '100%',
      width: 3,
      backgroundColor: 'white'
    }
  }));
};

var DraggableColBlock = function (props) {
  var selfRef = React.useRef(null);

  var _a = reactDnd.useDrag({
    item: {
      type: 'col',
      id: props.node.id
    },
    collect: function (monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    }
  }),
      isDragging = _a[0].isDragging,
      drag = _a[1];

  var _b = reactDnd.useDrop({
    accept: ['col', 'row', 'layer', 'custom', 'image', 'markdown'],
    canDrop: function (item) {
      var getNode = props.getNode,
          node = props.node;
      var parentNode = node.parentId ? getNode(node.parentId) : null;
      var draggedNodeType = item.type;
      return (// if our parent is a row, allow to place a sibling(col) before/after us
        draggedNodeType === 'col' && parentNode && parentNode.type === 'row' || // if we have children, we don't know if they can handle drops
        !props.children && (draggedNodeType === 'row' || draggedNodeType === 'markdown' || draggedNodeType === 'image' || draggedNodeType === 'layer' || draggedNodeType === 'custom')
      );
    },
    drop: function (item, monitor) {
      var isOver = monitor.isOver({
        shallow: true
      }); // ?TODO check if already dropped in nested target... instead of shallow isOver check?

      if (!isOver || !item) return null;

      if (item.type === 'col') {
        // geometry: figure out whether the dragged element should go after us or before us in parent container
        var placeBefore = shouldPlaceBefore('x');
        var anchorOpts = placeBefore ? {
          beforeItemId: props.node.id
        } : {
          afterItemId: props.node.id
        };
        props.changeBlocks(onDropped2(item.id, props.node.parentId || '', tslib_1.__assign({}, anchorOpts)));
      } else {
        var placeBefore = shouldPlaceBefore('y');
        var childrenIds = props.node.childrenIds;
        var numChildren = childrenIds.length;
        var anchorOpts = numChildren ? placeBefore ? {
          beforeItemId: childrenIds[0]
        } : {
          afterItemId: childrenIds[numChildren - 1]
        } : null;
        props.changeBlocks(onDropped2(item.id, props.node.id, tslib_1.__assign({}, anchorOpts)));
      }

      return {
        type: 'col',
        id: props.node.id
      };
    },
    collect: function (monitor) {
      var dragSourceItem = monitor.getItem();
      return {
        isOver: monitor.isOver({
          shallow: true
        }),
        initialClientOffset: monitor.getInitialClientOffset(),
        clientOffset: monitor.getClientOffset(),
        dragSourceItemType: dragSourceItem ? dragSourceItem.type : '',
        dragSourceItemId: dragSourceItem ? dragSourceItem.id : '',
        initialSourceClientOffset: monitor.getInitialSourceClientOffset()
      };
    }
  }),
      _c = _b[0],
      clientOffset = _c.clientOffset,
      dragSourceItemType = _c.dragSourceItemType,
      isOver = _c.isOver,
      drop = _b[1];

  var getBoundingRect = function () {
    return selfRef.current && selfRef.current.getBoundingClientRect ? selfRef.current.getBoundingClientRect() : null;
  };

  var shouldPlaceBefore = function (axis) {
    if (!clientOffset || !isOver) {
      return false;
    } // geometry: figure out whether the dragged element should go after us or before us


    var relativeDraggedPosition = getDragPositionRelativeToTarget2({
      clientX: clientOffset.x,
      clientY: clientOffset.y
    }, getBoundingRect());
    var placeBefore = relativeDraggedPosition && (axis === 'y' ? relativeDraggedPosition.top / relativeDraggedPosition.height < 0.45 : relativeDraggedPosition.left / relativeDraggedPosition.width < 0.45);
    return placeBefore;
  };

  var calcWantToPlaceNext = function (dragSourceItemType) {
    if (!dragSourceItemType || !isOver) return null;

    if (dragSourceItemType === 'col') {
      var placeBefore_1 = shouldPlaceBefore('x');
      return placeBefore_1 ? 'left' : 'right';
    }

    var placeBefore = shouldPlaceBefore('y');
    return placeBefore ? 'firstChild' : 'lastChild';
  };

  var wantToPlaceNext = calcWantToPlaceNext(dragSourceItemType);
  return React__default.createElement("div", {
    ref: selfRef,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React__default.createElement("div", {
    ref: drop,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React__default.createElement("div", {
    ref: drag,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React__default.createElement(ColBlock, tslib_1.__assign({}, props, {
    wantToPlaceNext: wantToPlaceNext,
    isDragging: isDragging
  })))));
};

var RowBlock =
/*#__PURE__*/

/** @class */
function (_super) {
  tslib_1.__extends(RowBlock, _super);

  function RowBlock() {
    return _super !== null && _super.apply(this, arguments) || this;
  }

  RowBlock.prototype.render = function () {
    var reactChildren = this.props.children;
    var _a = this.props,
        wantToPlaceNext = _a.wantToPlaceNext,
        node = _a.node,
        getNode = _a.getNode,
        undoRedoVersion = _a.undoRedoVersion,
        changeBlocks = _a.changeBlocks,
        renderEditBlock = _a.renderEditBlock,
        focusedNodeId = _a.focusedNodeId;
    var firstChildPlaceholderWidth = 3;
    var paddingLeft = (node.paddingLeftPercentWidth !== undefined ? node.paddingLeftPercentWidth : 0.05) * node.width;
    var paddingTop = (node.paddingTopPercentHeight !== undefined ? node.paddingTopPercentHeight : 0.05) * node.height;
    var runningWidth = wantToPlaceNext === 'firstChild' ? firstChildPlaceholderWidth : 0;
    return React.createElement("div", {
      key: 'row_' + node.id,
      style: {
        opacity: this.props.isDragging ? 0.5 : 1
      }
    }, wantToPlaceNext === 'top' && React.createElement("div", {
      style: {
        height: 3,
        width: '100%',
        backgroundColor: 'black'
      }
    }), React.createElement("div", {
      style: {
        position: 'relative',
        height: node.height,
        width: node.width,
        // padding: 20,
        borderStyle: 'dashed',
        borderWidth: 0,
        borderBottomWidth: 1,
        backgroundColor: node.isPlaceHolder ? 'darkgrey' : node.backgroundColor || '#b5bbb9b0'
      }
    }, wantToPlaceNext === 'firstChild' && React.createElement("div", {
      style: {
        height: '100%',
        width: firstChildPlaceholderWidth,
        backgroundColor: 'blue',
        position: 'absolute',
        top: paddingTop,
        left: paddingLeft,
        transform: "translate(" + runningWidth + "px,0)"
      }
    }), React.Children.count(reactChildren) ? reactChildren : node.childrenIds.map(function (id) {
      var node = getNode(id);
      if (!node) return null;
      var res = React.createElement("div", {
        className: "drag-node",
        key: 'node_' + node.id,
        style: {
          position: 'absolute',
          width: node.width,
          height: node.height,
          top: paddingTop,
          left: paddingLeft,
          transform: "translate(" + runningWidth + "px,0)"
        }
      }, renderEditBlock({
        node: node,
        getNode: getNode,
        undoRedoVersion: undoRedoVersion,
        focusedNodeId: focusedNodeId,
        renderEditBlock: renderEditBlock,
        changeBlocks: changeBlocks
      }));
      runningWidth += node.width;
      return res;
    }), wantToPlaceNext === 'lastChild' && React.createElement("div", {
      style: {
        position: 'absolute',
        top: paddingTop,
        left: paddingLeft,
        transform: "translate(" + runningWidth + "px,0)",
        height: '100%',
        width: firstChildPlaceholderWidth,
        backgroundColor: 'orange'
      }
    })), wantToPlaceNext === 'bottom' && React.createElement("div", {
      style: {
        height: 3,
        width: '100%',
        backgroundColor: 'green'
      }
    }));
  };

  return RowBlock;
}(React.Component);

var DraggableRowBlock = function (props) {
  var selfRef = React.useRef(null);

  var _a = reactDnd.useDrag({
    item: {
      type: 'row',
      id: props.node.id
    },
    collect: function (monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    }
  }),
      isDragging = _a[0].isDragging,
      drag = _a[1];

  var _b = reactDnd.useDrop({
    accept: ['col', 'row', 'layer', 'custom', 'image', 'markdown'],
    canDrop: function (item) {
      var getNode = props.getNode,
          node = props.node;
      var parentNode = node.parentId ? getNode(node.parentId) : null;
      var draggedNodeType = item.type;
      return (// if our parent is a col, allow to place a sibling (row) before/after us
        draggedNodeType === 'row' && parentNode && parentNode.type === 'col' || // if we have children, we don't know if they can handle drops
        !props.children && (draggedNodeType === 'col' || draggedNodeType === 'markdown' || draggedNodeType === 'image' || draggedNodeType === 'layer' || draggedNodeType === 'custom')
      );
    },
    drop: function (item, monitor) {
      var isOver = monitor.isOver({
        shallow: true
      }); // ?TODO check if already dropped in nested target... instead of shallow isOver check?

      if (!isOver || !item) return null;

      if (item.type === 'row') {
        // geometry: figure out whether the dragged element should go after us or before us in parent container
        var placeBefore = shouldPlaceBefore('y');
        var anchorOpts = placeBefore ? {
          beforeItemId: props.node.id
        } : {
          afterItemId: props.node.id
        };
        props.changeBlocks(onDropped2(item.id, props.node.parentId || '', tslib_1.__assign({}, anchorOpts)));
      } else {
        var placeBefore = shouldPlaceBefore('x');
        var childrenIds = props.node.childrenIds;
        var numChildren = childrenIds.length;
        var anchorOpts = numChildren ? placeBefore ? {
          beforeItemId: childrenIds[0]
        } : {
          afterItemId: childrenIds[numChildren - 1]
        } : null;
        props.changeBlocks(onDropped2(item.id, props.node.id, tslib_1.__assign({}, anchorOpts)));
      }

      return {
        type: 'row',
        id: props.node.id
      };
    },
    collect: function (monitor) {
      var dragSourceItem = monitor.getItem();
      return {
        isOver: monitor.isOver({
          shallow: true
        }),
        initialClientOffset: monitor.getInitialClientOffset(),
        clientOffset: monitor.getClientOffset(),
        dragSourceItemType: dragSourceItem ? dragSourceItem.type : '',
        dragSourceItemId: dragSourceItem ? dragSourceItem.id : '',
        initialSourceClientOffset: monitor.getInitialSourceClientOffset()
      };
    }
  }),
      _c = _b[0],
      clientOffset = _c.clientOffset,
      dragSourceItemType = _c.dragSourceItemType,
      isOver = _c.isOver,
      drop = _b[1];

  var getBoundingRect = function () {
    return selfRef.current && selfRef.current.getBoundingClientRect ? selfRef.current.getBoundingClientRect() : null;
  };

  var shouldPlaceBefore = function (axis) {
    if (!clientOffset || !isOver) {
      return false;
    } // geometry: figure out whether the dragged element should go after us or before us


    var relativeDraggedPosition = getDragPositionRelativeToTarget2({
      clientX: clientOffset.x,
      clientY: clientOffset.y
    }, getBoundingRect());
    var placeBefore = relativeDraggedPosition && (axis === 'y' ? relativeDraggedPosition.top / relativeDraggedPosition.height < 0.3 : relativeDraggedPosition.left / relativeDraggedPosition.width < 0.3);
    return placeBefore;
  };

  var calcWantToPlaceNext = function (dragSourceItemType) {
    if (!dragSourceItemType || !isOver) return null;

    if (dragSourceItemType === 'row') {
      var placeBefore_1 = shouldPlaceBefore('y');
      return placeBefore_1 ? 'top' : 'bottom';
    }

    var placeBefore = shouldPlaceBefore('x');
    return placeBefore ? 'firstChild' : 'lastChild';
  };

  var wantToPlaceNext = calcWantToPlaceNext(dragSourceItemType);
  return React.createElement("div", {
    ref: selfRef,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React.createElement("div", {
    ref: drop,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React.createElement("div", {
    ref: drag,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React.createElement(RowBlock, tslib_1.__assign({}, props, {
    wantToPlaceNext: wantToPlaceNext,
    isDragging: isDragging
  })))));
};

var ImageBlockComp =
/*#__PURE__*/

/** @class */
function (_super) {
  tslib_1.__extends(ImageBlockComp, _super);

  function ImageBlockComp() {
    var _this = _super !== null && _super.apply(this, arguments) || this;

    _this.selfRef = null;

    _this.getBoundingRect = function () {
      return _this.selfRef && _this.selfRef.getBoundingClientRect ? _this.selfRef.getBoundingClientRect() : null;
    };

    return _this;
  }

  ImageBlockComp.prototype.render = function () {
    var _this = this;

    var node = this.props.node;
    return React.createElement("div", {
      ref: function (el) {
        return _this.selfRef = el;
      },
      // draggable
      // onDragStart={e => onDragStart(e, this.props.node, this.getBoundingRect)}
      style: {
        opacity: this.props.isDragging ? 0.5 : 1,
        width: '100%',
        height: '100%',
        backgroundColor: 'violet',
        padding: 5,
        borderRadius: 3
      }
    }, React.createElement("img", {
      style: {
        width: '100%',
        height: '100%'
      },
      src: node.value
    }));
  };

  return ImageBlockComp;
}(React.Component);

var ImageBlock = function (props) {
  var _a = reactDnd.useDrag({
    item: {
      type: 'image',
      id: props.node.id
    },
    collect: function (monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    }
  }),
      isDragging = _a[0].isDragging,
      drag = _a[1];

  return React.createElement("div", {
    ref: drag,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React.createElement(ImageBlockComp, tslib_1.__assign({}, props, {
    isDragging: isDragging
  })));
};

var MarkdownBlockComp =
/*#__PURE__*/

/** @class */
function (_super) {
  tslib_1.__extends(MarkdownBlockComp, _super);

  function MarkdownBlockComp() {
    var _this = _super !== null && _super.apply(this, arguments) || this;

    _this.selfRef = null;

    _this.getBoundingRect = function () {
      return _this.selfRef && _this.selfRef.getBoundingClientRect ? _this.selfRef.getBoundingClientRect() : null;
    };

    return _this;
  }

  MarkdownBlockComp.prototype.render = function () {
    var _this = this;

    var _a = this.props,
        node = _a.node,
        update = _a.update;
    var value = node.value;
    return React.createElement("div", {
      ref: function (el) {
        return _this.selfRef = el;
      },
      // draggable
      // onDragStart={e => onDragStart(e, this.props.node, this.getBoundingRect)}
      style: {
        opacity: this.props.isDragging ? 0.5 : 1,
        width: '100%',
        height: '100%',
        backgroundColor: '#ffc0cb75',
        padding: 5,
        borderRadius: 3
      }
    }, React.createElement("textarea", {
      className: "form-control",
      name: "value",
      placeholder: "Type here...",
      value: value,
      style: {
        width: '95%',
        height: '95%',
        outline: 'none'
      },
      onChange: function (e) {
        return update(node.id, {
          value: e.target.value
        });
      }
    }));
  };

  return MarkdownBlockComp;
}(React.Component);

var MarkdownBlock = function (props) {
  var _a = reactDnd.useDrag({
    item: {
      type: 'markdown',
      id: props.node.id
    },
    collect: function (monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    }
  }),
      isDragging = _a[0].isDragging,
      drag = _a[1];

  return React.createElement("div", {
    ref: drag,
    style: {
      width: '100%',
      height: '100%'
    }
  }, React.createElement(MarkdownBlockComp, tslib_1.__assign({}, props, {
    isDragging: isDragging
  })));
};

var ResizableBlock = function (props) {
  var node = props.node,
      renderEditBlock = props.renderEditBlock,
      getNode = props.getNode,
      undoRedoVersion = props.undoRedoVersion,
      changeBlocks = props.changeBlocks,
      focusedNodeId = props.focusedNodeId,
      width = props.width,
      height = props.height;
  return React.createElement("div", {
    onClick: function (e) {
      e.stopPropagation();
      changeBlocks(function (value) {
        return focusNode(value, node);
      });
    },
    style: node.id === focusedNodeId ? {
      borderStyle: 'dashed',
      borderWidth: 1,
      borderColor: 'orange'
    } : {}
  }, React.createElement(reactResizable.ResizableBox, {
    className: "box",
    width: width,
    height: height,
    onResizeStart: function (event) {
      event.preventDefault();
      event.stopPropagation();
    },
    onResize: function (_event, _a) {
      var size = _a.size;
      var width = size.width,
          height = size.height;
      changeBlocks(function (value) {
        return update(value, node.id, {
          width: width,
          height: height
        });
      });
    }
  }, node.type === 'col' ? React.createElement(DraggableColBlock, {
    key: 'col_' + node.id,
    node: node,
    renderEditBlock: renderEditBlock,
    getNode: getNode,
    changeBlocks: changeBlocks,
    focusedNodeId: focusedNodeId,
    undoRedoVersion: undoRedoVersion
  }) : node.type === 'row' ? React.createElement(DraggableRowBlock, {
    key: 'row_' + node.id,
    node: node,
    renderEditBlock: renderEditBlock,
    getNode: getNode,
    changeBlocks: changeBlocks,
    focusedNodeId: focusedNodeId,
    undoRedoVersion: undoRedoVersion
  }) : node.type === 'markdown' ? React.createElement(MarkdownBlock, {
    node: node,
    update: function (nodeId, props) {
      return changeBlocks(function (value) {
        return update(value, nodeId, props);
      });
    }
  }) : node.type === 'layer' ? React.createElement(AbsoluteLayerBlock, {
    key: 'layer_' + node.id,
    node: node,
    renderEditBlock: renderEditBlock,
    getNode: getNode,
    changeBlocks: changeBlocks,
    focusedNodeId: focusedNodeId,
    undoRedoVersion: undoRedoVersion
  }) : React.createElement(ImageBlock, {
    key: 'image_' + node.id,
    node: node,
    renderEditBlock: renderEditBlock,
    getNode: getNode,
    changeBlocks: changeBlocks,
    focusedNodeId: focusedNodeId,
    undoRedoVersion: undoRedoVersion
  })));
};
var defaultRenderEditBlock = function (_a) {
  var node = _a.node,
      renderEditBlock = _a.renderEditBlock,
      getNode = _a.getNode,
      changeBlocks = _a.changeBlocks,
      undoRedoVersion = _a.undoRedoVersion,
      focusedNodeId = _a.focusedNodeId;
  return React.createElement(ResizableBlock, {
    width: node.width,
    height: node.height,
    renderEditBlock: renderEditBlock,
    node: node,
    getNode: getNode,
    changeBlocks: changeBlocks,
    focusedNodeId: focusedNodeId,
    undoRedoVersion: undoRedoVersion
  });
};

var maxUndoStack = 10;

var BlockEditor =
/*#__PURE__*/

/** @class */
function (_super) {
  tslib_1.__extends(BlockEditor, _super);

  function BlockEditor() {
    var _this = _super !== null && _super.apply(this, arguments) || this;

    _this.haveHtml5DndAPI = haveDraggableHtml5API();
    _this.state = {
      value: _this.props.value
    };

    _this.getNode = function (id) {
      return _this.state.value.byId[id];
    };

    _this.onSendOp = function (op) {
      var value = op(_this.state.value);

      _this.setState({
        value: value
      });
    };

    return _this;
  }

  BlockEditor.prototype.componentDidMount = function () {
    if (this.props.onChange) {
      // Notify parent of change
      var value = this.state.value;
      this.props.onChange(value);
    }
  };

  BlockEditor.prototype.componentDidUpdate = function (prevProps, prevState) {
    if (this.props.onChange && this.state.value !== prevState.value && this.state.value !== this.props.value) {
      var value = this.state.value;
      this.props.onChange(value);
    } else if (this.props.value && prevProps.value !== this.props.value && this.props.value !== this.state.value) {
      this.setState({
        value: this.props.value
      });
    } // make the last byId reachable through undo


    if (this.state.value.byId !== prevState.value.byId && this.state.value.undoStack === prevState.value.undoStack) {
      this.setState({
        value: tslib_1.__assign({}, this.state.value, {
          undoStack: [lodash.cloneDeep(prevState.value.byId)].concat(this.state.value.undoStack).slice(0, maxUndoStack)
        })
      });
    }
  };

  BlockEditor.prototype.render = function () {
    var renderEditBlock = this.props.renderEditBlock;
    var _a = this.state.value,
        byId = _a.byId,
        rootNodeId = _a.rootNodeId,
        undoRedoVersion = _a.undoRedoVersion;
    var rootNode = byId[rootNodeId];
    var Backend = this.haveHtml5DndAPI ? Html5Backend : TouchBackend; // console.log('LAYOUT:', JSON.stringify(byId), JSON.stringify(rootNode));

    return React.createElement("div", {
      style: {
        position: 'relative',
        width: rootNode.width,
        height: rootNode.height
      }
    }, React.createElement(reactDnd.DndProvider, {
      backend: Backend
    }, React.createElement(DraggableColBlock, {
      key: 'col_' + rootNode.id,
      undoRedoVersion: undoRedoVersion,
      node: rootNode,
      getNode: this.getNode,
      changeBlocks: this.onSendOp,
      focusedNodeId: this.state.value.focusedNodeId,
      renderEditBlock: renderEditBlock
    })));
  };

  BlockEditor.defaultProps = {
    renderEditBlock: defaultRenderEditBlock,
    onChange: function (_v) {}
  };
  return BlockEditor;
}(React.Component);

function haveDraggableHtml5API() {
  var iOS = !!navigator.userAgent.match('iPhone OS') || !!navigator.userAgent.match('iPad');
  if (iOS) return false;
  var div = window.document.createElement('div');
  return 'draggable' in div || 'ondragstart' in div && 'ondrop' in div;
}

function hasBorder(node) {
  return !!node.borderWidth && node.borderWidth > 0 || !!node.borderTopWidth && node.borderTopWidth > 0 || !!node.borderBottomWidth && node.borderBottomWidth > 0 || !!node.borderLeftWidth && node.borderLeftWidth > 0 || !!node.borderRightWidth && node.borderRightWidth > 0;
}

var copyBasicProps = function (node) {
  return {
    color: node.color || undefined,
    display: node.display || undefined,
    flexDirection: node.flexDirection || undefined,
    justifyContent: node.justifyContent || undefined,
    alignItems: node.alignItems || undefined,
    backgroundColor: node.backgroundColor || undefined,
    borderWidth: node.borderWidth !== undefined ? node.borderWidth : 0,
    borderTopWidth: node.borderTopWidth !== undefined ? node.borderTopWidth : undefined,
    borderBottomWidth: node.borderBottomWidth !== undefined ? node.borderBottomWidth : undefined,
    borderLeftWidth: node.borderLeftWidth !== undefined ? node.borderLeftWidth : undefined,
    borderRightWidth: node.borderRightWidth !== undefined ? node.borderRightWidth : undefined,
    borderStyle: !node.borderStyle ? hasBorder(node) ? 'solid' : undefined : node.borderStyle || undefined
  };
};

var Preview = function (_a) {
  var byId = _a.byId,
      node = _a.node,
      _b = _a.renderPreviewBlock,
      renderPreviewBlock = _b === void 0 ? defaultRenderPreviewBlock : _b;
  if (!node) return null;

  switch (node.type) {
    case 'markdown':
      return React.createElement("div", {
        key: node.id,
        style: tslib_1.__assign({}, copyBasicProps(node), {
          height: node.height,
          width: node.width
        })
      }, React.createElement(MarkDown, {
        source: node.value
      }));

    case 'layer':
      return React.createElement("div", {
        key: node.id,
        style: tslib_1.__assign({
          // flex: 1,
          position: 'relative'
        }, copyBasicProps(node), {
          height: node.height,
          width: node.width
        })
      }, node.childrenIds.map(function (id) {
        var childNode = byId[id];
        if (!childNode) return React.createElement("div", null, "Error in layer ", node.id, ": child node ", id, " not found");
        return React.createElement("div", {
          key: childNode.id,
          style: {
            position: 'absolute',
            top: childNode.top || 0,
            left: childNode.left || 0
          }
        }, renderPreviewBlock({
          byId: byId,
          node: childNode,
          renderPreviewBlock: renderPreviewBlock
        }));
      }));

    case 'image':
      return React.createElement("div", {
        key: node.id,
        style: {
          height: '100%',
          width: '100%'
        }
      }, React.createElement("img", {
        src: node.value
      }));

    case 'col':
      return React.createElement("div", {
        key: node.id,
        style: tslib_1.__assign({}, copyBasicProps(node), {
          width: node.width,
          height: node.height,
          display: 'flex',
          flexDirection: 'column'
        })
      }, node.childrenIds.map(function (id) {
        var childNode = byId[id];
        if (!childNode) return React.createElement("div", null, "Error in col ", node.id, ": child node ", id, " not found");
        return renderPreviewBlock({
          byId: byId,
          node: childNode,
          renderPreviewBlock: renderPreviewBlock
        });
      }));

    case 'row':
      return React.createElement("div", {
        key: node.id,
        style: tslib_1.__assign({}, copyBasicProps(node), {
          width: node.width,
          height: node.height,
          display: 'flex',
          flexDirection: 'row'
        })
      }, node.childrenIds.map(function (id) {
        var childNode = byId[id];
        if (!childNode) return React.createElement("div", null, "Error in row ", node.id, ": child node ", id, " not found");
        return renderPreviewBlock({
          byId: byId,
          node: childNode,
          renderPreviewBlock: renderPreviewBlock
        });
      }));

    case 'custom':
      return null;
  }
};
var defaultRenderPreviewBlock = function (_a) {
  var byId = _a.byId,
      node = _a.node,
      renderPreviewBlock = _a.renderPreviewBlock;
  return React.createElement(Preview, {
    key: 'node_' + node.id,
    node: node,
    byId: byId,
    renderPreviewBlock: renderPreviewBlock
  });
};

var BlockBreadCrumbs = function (props) {
  var byId = props.byId,
      node = props.node,
      onSelect = props.onSelect,
      _a = props.navClassName,
      navClassName = _a === void 0 ? 'breadcrumb' : _a,
      _b = props.itemClassName,
      itemClassName = _b === void 0 ? 'breadcrumb-item btn btn-link active' : _b;
  if (!node) return null;
  var crumbs = [];
  var id = node.id;

  while (id) {
    var node_1 = byId[id];
    if (!node_1) break;
    crumbs.unshift({
      label: node_1.type + "-" + node_1.id,
      id: node_1.id
    });
    id = node_1.parentId || '';
  }

  return React.createElement("nav", {
    "aria-label": "breadcrumb"
  }, React.createElement("ol", {
    className: navClassName
  }, crumbs.map(function (crumb) {
    return React.createElement("li", {
      onClick: function () {
        return onSelect(crumb.id);
      },
      key: 'crumb_' + crumb.id,
      className: itemClassName,
      "aria-current": "page"
    }, crumb.label);
  })));
};

var nestedElementPercentWidthHeight = 0.95;
function addRow(value, props) {
  var byId = value.byId,
      rootNodeId = value.rootNodeId;
  var focusedNodeId = value.focusedNodeId;
  var parentNodeId = (props ? props.parentId : focusedNodeId) || rootNodeId;
  var parentNode = byId[parentNodeId];
  var allowedParents = ['col', 'layer'];

  if (!parentNode || allowedParents.indexOf(parentNode.type) < 0) {
    // try parent
    parentNodeId = parentNode ? parentNode.parentId || '' : '';
    parentNode = parentNodeId ? byId[parentNodeId] : parentNode;

    if (!parentNode || allowedParents.indexOf(parentNode.type) < 0) {
      return {
        error: 'Can only add a row inside ' + allowedParents.join(', ') + ', please select a column by clicking on it',
        value: value
      };
    }
  }

  var newNodeId = newBlockId(byId, 'row');

  var row = tslib_1.__assign({
    type: 'row',
    height: 100,
    width: parentNode && parentNode.width ? parentNode.width * nestedElementPercentWidthHeight : 500,
    paddingLeftPercentWidth: (1 - nestedElementPercentWidthHeight) / 2,
    paddingTopPercentHeight: (1 - nestedElementPercentWidthHeight) / 2
  }, props, {
    id: newNodeId,
    name: newNodeId,
    childrenIds: []
  }); // add child


  byId = placeNodeInParent(byId, row, parentNode.id);
  parentNode = byId[parentNodeId]; // update new child and all siblings' heights

  var updates = {};

  if (parentNode && parentNode.type === 'col') {
    if (parentNode && parentNode.childrenIds.length >= 1) {
      var childHeight = Math.max(20, parentNode.height / parentNode.childrenIds.length * nestedElementPercentWidthHeight);

      for (var _i = 0, _a = parentNode.childrenIds; _i < _a.length; _i++) {
        var childId = _a[_i];
        updates[childId] = tslib_1.__assign({}, byId[childId], {
          height: childHeight
        });
      }
    }
  }

  return {
    createdBlock: row,
    value: tslib_1.__assign({}, value, {
      byId: tslib_1.__assign({}, byId, updates),
      focusedNodeId: focusedNodeId || row.id
    })
  };
}
function addCol(value, props) {
  var byId = value.byId,
      rootNodeId = value.rootNodeId;
  var focusedNodeId = value.focusedNodeId;
  var parentNodeId = (props ? props.parentId : focusedNodeId) || rootNodeId;
  var parentNode = byId[parentNodeId];
  var allowedParents = ['row', 'layer'];

  if (!parentNode || allowedParents.indexOf(parentNode.type) < 0) {
    // try parent
    parentNodeId = parentNode ? parentNode.parentId || '' : '';
    parentNode = parentNodeId ? byId[parentNodeId] : parentNode;

    if (!parentNode || allowedParents.indexOf(parentNode.type) < 0) {
      return {
        error: 'Can only add a column inside ' + allowedParents.join(', ') + ', please select a column by clicking on it',
        value: value
      };
    }
  }

  var newNodeId = newBlockId(byId, 'col');

  var col = tslib_1.__assign({
    type: 'col',
    width: 100,
    height: parentNode.height ? parentNode.height * nestedElementPercentWidthHeight : 100,
    paddingLeftPercentWidth: (1 - nestedElementPercentWidthHeight) / 2,
    paddingTopPercentHeight: (1 - nestedElementPercentWidthHeight) / 2
  }, props, {
    id: newNodeId,
    name: newNodeId,
    childrenIds: []
  }); // add child


  byId = placeNodeInParent(byId, col, parentNodeId);
  parentNode = byId[parentNodeId]; // update new child and all siblings' widths

  var updates = {};

  if (parentNode && parentNode.type === 'row') {
    if (parentNode && parentNode.childrenIds.length >= 1) {
      var childWidth = Math.max(20, parentNode.width / parentNode.childrenIds.length * nestedElementPercentWidthHeight);

      for (var _i = 0, _a = parentNode.childrenIds; _i < _a.length; _i++) {
        var childId = _a[_i];
        updates[childId] = tslib_1.__assign({}, byId[childId], {
          width: childWidth
        });
      }
    }
  }

  return {
    createdBlock: col,
    value: tslib_1.__assign({}, value, {
      byId: tslib_1.__assign({}, byId, updates),
      focusedNodeId: focusedNodeId || col.id
    })
  };
}
function addMarkDown(value, props) {
  var byId = value.byId,
      rootNodeId = value.rootNodeId;
  var focusedNodeId = value.focusedNodeId;
  var parentNodeId = (props ? props.parentId : focusedNodeId) || rootNodeId;
  var parentNode = byId[parentNodeId];
  var allowedParents = ['row', 'col', 'layer'];

  if (!parentNode || allowedParents.indexOf(parentNode.type) < 0) {
    return {
      error: 'Can only add markdown text inside ' + allowedParents.join(', '),
      value: value
    };
  }

  var newNodeId = newBlockId(byId, 'markdown');

  var markdown = tslib_1.__assign({
    type: 'markdown',
    width: Math.max(1, parentNode.width - 20),
    height: Math.max(1, parentNode.height - 20)
  }, props, {
    id: newNodeId,
    name: newNodeId,
    childrenIds: []
  });

  return {
    createdBlock: markdown,
    value: tslib_1.__assign({}, value, {
      byId: placeNodeInParent(byId, markdown, parentNodeId),
      focusedNodeId: markdown.id
    })
  };
}
function addImage(value, props) {
  var url = props && props.value ? props.value : window.prompt('Please enter image url', 'https://lorempixel.com/200/200/');
  if (url === null) return {
    error: 'no image selected',
    value: value
  };
  var byId = value.byId,
      rootNodeId = value.rootNodeId;
  var focusedNodeId = value.focusedNodeId;
  var parentNodeId = (props ? props.parentId : focusedNodeId) || rootNodeId;
  var parentNode = byId[parentNodeId];
  var allowedParents = ['row', 'col', 'layer'];

  if (!parentNode || allowedParents.indexOf(parentNode.type) < 0) {
    return {
      error: 'Can only add an image inside ' + allowedParents.join(', '),
      value: value
    };
  }

  var newNodeId = newBlockId(byId, 'image');

  var image = tslib_1.__assign({
    type: 'image',
    value: url,
    width: 200,
    height: 200
  }, props, {
    id: newNodeId,
    name: newNodeId,
    childrenIds: []
  });

  return {
    createdBlock: image,
    value: tslib_1.__assign({}, value, {
      byId: placeNodeInParent(byId, image, parentNodeId),
      focusedNodeId: image.id
    })
  };
}
function addLayer(value, props) {
  var byId = value.byId,
      rootNodeId = value.rootNodeId;
  var focusedNodeId = value.focusedNodeId;
  var parentNodeId = (props ? props.parentId : focusedNodeId) || rootNodeId;
  var parentNode = byId[parentNodeId];
  var allowedParents = ['row', 'col', 'layer'];

  if (!parentNode || allowedParents.indexOf(parentNode.type) < 0) {
    return {
      error: 'Can only add a layer inside ' + allowedParents.join(', '),
      value: value
    };
  }

  var newNodeId = newBlockId(byId, 'layer');

  var layer = tslib_1.__assign({
    type: 'layer',
    width: Math.max(1, parentNode.width * nestedElementPercentWidthHeight),
    height: Math.max(1, parentNode.height * nestedElementPercentWidthHeight),
    paddingLeftPercentWidth: (1 - nestedElementPercentWidthHeight) / 2,
    paddingTopPercentHeight: (1 - nestedElementPercentWidthHeight) / 2
  }, props, {
    id: newNodeId,
    name: newNodeId,
    childrenIds: []
  });

  return {
    createdBlock: layer,
    value: tslib_1.__assign({}, value, {
      byId: placeNodeInParent(byId, layer, parentNodeId),
      focusedNodeId: layer.id
    })
  };
}

var BlockEditorControlDefaultUI =
/*#__PURE__*/

/** @class */
function (_super) {
  tslib_1.__extends(BlockEditorControlDefaultUI, _super);

  function BlockEditorControlDefaultUI() {
    var _this = _super !== null && _super.apply(this, arguments) || this;

    _this.state = {
      selectedMenu: null,
      selectedColor: null
    };

    _this.toggleMenu = function (menu) {
      _this.setState({
        selectedMenu: _this.state.selectedMenu === menu ? null : menu
      });

      if (menu === 'color' || menu === 'backgroundColor') {
        var _a = _this.props.value,
            byId = _a.byId,
            focusedNodeId = _a.focusedNodeId;
        var selectedNode = focusedNodeId ? byId[focusedNodeId] : null;

        _this.setState({
          selectedColor: selectedNode ? selectedNode.color || null : null
        });
      }
    };

    _this.renderColorMenuItem = function (which) {
      var _a = _this.props.value,
          byId = _a.byId,
          focusedNodeId = _a.focusedNodeId;
      if (!focusedNodeId) return;
      var selectedNode = focusedNodeId ? byId[focusedNodeId] : null;
      return React__default.createElement(React__default.Fragment, null, React__default.createElement(ColorButton, {
        onClick: function () {
          return _this.toggleMenu(which);
        },
        color: selectedNode ? selectedNode[which] || '' : ''
      }), _this.state.selectedMenu === which && React__default.createElement("div", {
        style: {
          position: 'absolute',
          zIndex: 100,
          backgroundColor: 'white'
        }
      }, React__default.createElement("button", {
        className: _this.props.buttonClassName,
        onClick: function () {
          var _a;

          _this.props.updateBlock(focusedNodeId, (_a = {}, _a[which] = undefined, _a));

          _this.toggleMenu(which);
        }
      }, "Clear"), React__default.createElement("button", {
        className: _this.props.buttonClassName,
        onClick: function () {
          var _a;

          _this.toggleMenu(which);

          _this.props.updateBlock(focusedNodeId, (_a = {}, _a[which] = _this.state.selectedColor, _a));
        }
      }, "Done"), React__default.createElement(reactColor.SketchPicker, {
        color: _this.state.selectedColor || undefined,
        onChangeComplete: function (_a) {
          var hex = _a.hex;

          _this.setState({
            selectedColor: hex
          });
        }
      })));
    };

    return _this;
  }

  BlockEditorControlDefaultUI.prototype.render = function () {
    var _a = this.props,
        buttonClassName = _a.buttonClassName,
        breadCrumbsProps = _a.breadCrumbsProps,
        _b = _a.value,
        byId = _b.byId,
        focusedNodeId = _b.focusedNodeId;
    var props = this.props;
    var focusedNode = focusedNodeId ? byId[focusedNodeId] : null;
    var focusNode = props.focusNode;

    var handleBlockChange = function (nodeId, prop, numeric) {
      return function (event) {
        var _a;

        props.updateBlock(nodeId, (_a = {}, _a[prop] = numeric ? event.target.value ? parseInt(event.target.value, 10) : undefined : event.target.value, _a));
      };
    };

    var blockProps = [{
      prop: 'width',
      numeric: true
    }, {
      prop: 'height',
      numeric: true
    }, {
      prop: 'top',
      numeric: true
    }, {
      prop: 'left',
      numeric: true
    }, {
      prop: 'borderWidth',
      numeric: true
    }, {
      prop: 'borderTopWidth',
      numeric: true
    }, {
      prop: 'borderBottomWidth',
      numeric: true
    }, {
      prop: 'borderLeftWidth',
      numeric: true
    }, {
      prop: 'borderRightWidth',
      numeric: true
    }, {
      prop: 'borderStyle',
      numeric: false
    }];
    var btnCls = buttonClassName;
    return React__default.createElement("div", null, React__default.createElement("div", null, React__default.createElement("button", {
      "aria-label": "add row",
      className: btnCls,
      onClick: function () {
        return props.addRow();
      }
    }, "+ Row"), React__default.createElement("button", {
      "aria-label": "add column",
      className: btnCls,
      onClick: function () {
        return props.addCol();
      }
    }, "+ Col"), React__default.createElement("button", {
      "aria-label": "add layer",
      className: btnCls,
      onClick: function () {
        return props.addLayer();
      }
    }, "+ Layer"), React__default.createElement("button", {
      "aria-label": "add markdown",
      className: btnCls,
      onClick: function () {
        return props.addMarkDown();
      }
    }, "+ Text"), React__default.createElement("button", {
      "aria-label": "add image",
      className: btnCls,
      onClick: function () {
        return props.addImage();
      }
    }, "+ Image"), React__default.createElement("button", {
      "aria-label": "add image",
      className: btnCls,
      onClick: function () {
        props.onChange(addTable(props.value, props.value.focusedNodeId));
      }
    }, "+ Table"), React__default.createElement("button", {
      "aria-label": "undo",
      className: btnCls,
      onClick: props.undo
    }, "Undo"), React__default.createElement("button", {
      "aria-label": "redo",
      className: btnCls,
      onClick: props.redo
    }, "Redo")), focusedNode && React__default.createElement("div", {
      style: {
        paddingLeft: 15
      }
    }, this.renderColorMenuItem('color'), this.renderColorMenuItem('backgroundColor'), React__default.createElement("button", {
      className: btnCls,
      onClick: function () {
        return props.moveInDirection(focusedNode.id, 'left');
      }
    }, "\u2B05\uFE0F"), React__default.createElement("button", {
      className: btnCls,
      onClick: function () {
        return props.moveInDirection(focusedNode.id, 'right');
      }
    }, "\u27A1\uFE0F"), React__default.createElement("button", {
      className: btnCls,
      onClick: function () {
        return props.moveInDirection(focusedNode.id, 'up');
      }
    }, "\u2B06"), React__default.createElement("button", {
      className: btnCls,
      onClick: function () {
        return props.moveInDirection(focusedNode.id, 'down');
      }
    }, "\u2B07"), React__default.createElement("button", {
      className: btnCls,
      onClick: function () {
        return props.onChange(flipToFront(props.value, props.value.focusedNodeId));
      }
    }, "FlipToFront"), React__default.createElement("button", {
      "aria-label": "copy",
      className: btnCls,
      onClick: function () {
        return props.copyFocused();
      }
    }, "Copy"), React__default.createElement("button", {
      "aria-label": "paste",
      className: btnCls,
      onClick: function () {
        return props.paste();
      }
    }, "Paste"), React__default.createElement("button", {
      "aria-label": "delete",
      onClick: props.removeFocused
    }, "\uD83D\uDDD1"), React__default.createElement("div", null, React__default.createElement(BlockBreadCrumbs, tslib_1.__assign({
      onSelect: function (nodeId) {
        return focusNode(nodeId, true);
      },
      byId: byId,
      node: focusedNode
    }, breadCrumbsProps))), React__default.createElement("div", {
      style: {
        paddingBottom: 20
      }
    }, blockProps.map(function (blockProp) {
      return React__default.createElement("div", {
        key: blockProp.prop,
        style: {
          display: 'inline-block',
          marginRight: 10
        },
        className: "form-group"
      }, React__default.createElement("label", {
        style: {
          marginRight: 5
        },
        htmlFor: blockProp.prop,
        className: "form-label"
      }, blockProp.prop), React__default.createElement("input", {
        name: blockProp.prop,
        className: "form-control",
        key: 'prop_' + blockProp.prop,
        value: focusedNode[blockProp.prop] === undefined ? '' : focusedNode[blockProp.prop],
        onChange: handleBlockChange(focusedNode.id, blockProp.prop, blockProp.numeric),
        type: blockProp.numeric ? 'numeric' : undefined
      }));
    }))));
  };

  return BlockEditorControlDefaultUI;
}(React__default.Component);

var ColorButton = function (props) {
  return React__default.createElement("button", {
    style: {
      borderWidth: 1,
      fontSize: 11,
      borderStyle: 'solid',
      justifyContent: 'flex-start'
    },
    onClick: props.onClick
  }, React__default.createElement("div", {
    style: {
      display: 'flex'
    }
  }, React__default.createElement("div", {
    style: {
      borderWidth: 1,
      borderRadius: 3,
      borderStyle: 'solid',
      width: 20,
      height: 20,
      backgroundColor: props.color || undefined
    }
  }), "\u00A0", !props.color ? 'none' : ''));
};

function flipToFront(value, nodeId) {
  if (!nodeId) return value;
  var byId = value.byId;
  var node = byId[nodeId];
  if (!node) return value;
  var parentId = node.parentId;
  var parentNode = parentId ? byId[parentId] : undefined;
  if (!parentNode || !parentId) return value;
  return move(value, nodeId, parentId, {
    afterItemId: parentNode.childrenIds[parentNode.childrenIds.length - 1]
  });
}

function addTable(value, parentId) {
  if (!parentId) {
    alert('Please select node to add table to');
    return value;
  }

  var numRowsStr = window.prompt('Enter number or rows (max 10)', '5');
  if (numRowsStr === null) return value;
  var numColsStr = window.prompt('Enter number or columns (max 10)', '5');
  if (numColsStr === null) return value;
  var widthStr = window.prompt('Enter table width (max 400)', '300');
  if (widthStr === null) return value;
  var heightStr = window.prompt('Enter table height (max 400)', '300');
  if (heightStr === null) return value;
  var width = Math.min(parseInt(widthStr, 10), 400);
  var height = Math.min(parseInt(heightStr, 10), 400);

  var _a = addCol(value, {
    width: width,
    height: height,
    parentId: parentId
  }),
      newValue = _a.value,
      table = _a.createdBlock;

  if (!table) return value;
  value = newValue;
  var tableId = table.id;
  var numRows = Math.max(1, Math.min(parseInt(numRowsStr, 10), 10));
  var numCols = Math.max(1, Math.min(parseInt(numColsStr, 10), 10));

  for (var i = 0; i < numRows; ++i) {
    var _b = addRow(value, {
      height: height / numRows,
      parentId: tableId
    }),
        newValue_1 = _b.value,
        row = _b.createdBlock;

    if (!row) break;
    value = newValue_1;

    for (var j = 0; j < numCols; ++j) {
      var _c = addCol(value, {
        parentId: row.id,
        height: row.height,
        width: row.width / numCols
      }),
          newValue_2 = _c.value,
          col = _c.createdBlock;

      if (!col) break;
      value = newValue_2;
    }
  }

  return value;
}

var BlockEditorControl =
/*#__PURE__*/

/** @class */
function (_super) {
  tslib_1.__extends(BlockEditorControl, _super);

  function BlockEditorControl() {
    var _this = _super !== null && _super.apply(this, arguments) || this;

    _this.addRow = function (props) {
      var _a = addRow(_this.props.value, props),
          error = _a.error,
          value = _a.value;

      if (error) {
        alert(error);
        return;
      }

      _this.props.onChange(value);
    };

    _this.addCol = function (props) {
      var _a = addCol(_this.props.value, props),
          error = _a.error,
          value = _a.value;

      if (error) {
        alert(error);
        return;
      }

      _this.props.onChange(value);
    };

    _this.addMarkDown = function (props) {
      var _a = addMarkDown(_this.props.value, props),
          error = _a.error,
          value = _a.value;

      if (error) {
        alert(error);
        return;
      }

      _this.props.onChange(value);
    };

    _this.addImage = function (props) {
      var _a = addImage(_this.props.value, props),
          error = _a.error,
          value = _a.value;

      if (error) {
        alert(error);
        return;
      }

      _this.props.onChange(value);
    };

    _this.addLayer = function (props) {
      var _a = addLayer(_this.props.value, props),
          error = _a.error,
          value = _a.value;

      if (error) {
        alert(error);
        return;
      }

      _this.props.onChange(value);
    };

    _this.removeFocused = function () {
      var focusedNodeId = _this.props.value.focusedNodeId;

      if (focusedNodeId) {
        _this.destroy(focusedNodeId);
      }
    };

    _this.focusNode = function (node, focus) {
      if (focus === void 0) {
        focus = true;
      }

      var value = _this.props.value;

      _this.props.onChange(focusNode(value, node, focus));
    };

    _this.create = function (props) {
      _this.props.onChange(create(_this.props.value, props));
    };

    _this.updateBlock = function (nodeId, propsToUpdate) {
      _this.props.onChange(update(_this.props.value, nodeId, propsToUpdate));
    };

    _this.destroy = function (nodeId) {
      _this.props.onChange(destroy(_this.props.value, nodeId));
    };

    _this.move = function (nodeId, targetParentId, opts) {
      if (opts === void 0) {
        opts = {};
      }

      _this.props.onChange(move(_this.props.value, nodeId, targetParentId, opts));
    };

    _this.moveInDirection = function (nodeId, direction) {
      _this.props.onChange(moveInDirection(_this.props.value, nodeId, {
        direction: direction
      }));
    };

    _this.copyFocused = function () {
      var _a = _this.props.value,
          byId = _a.byId,
          focusedNodeId = _a.focusedNodeId;

      if (focusedNodeId) {
        var focusedNode = byId[focusedNodeId];
        focusedNode && _this.props.onChange(tslib_1.__assign({}, _this.props.value, {
          copiedNode: deepCopy(byId, focusedNode)
        }));
      }
    };

    _this.paste = function () {
      _this.props.onChange(paste(_this.props.value));
    };

    _this.undo = function () {
      var _a = _this.props.value,
          byId = _a.byId,
          undoStack = _a.undoStack,
          redoStack = _a.redoStack;

      if (undoStack.length > 0) {
        var newById = undoStack[0];
        var newRedoStack = [byId].concat(redoStack);
        var newUndoStack = undoStack.slice(1);
        var undoRedoVersion = _this.props.value.undoRedoVersion;

        _this.props.onChange(tslib_1.__assign({}, _this.props.value, {
          undoRedoVersion: undoRedoVersion + 1,
          byId: newById,
          undoStack: newUndoStack,
          redoStack: newRedoStack
        }));
      }
    };

    _this.redo = function () {
      var _a = _this.props.value,
          byId = _a.byId,
          undoStack = _a.undoStack,
          redoStack = _a.redoStack;

      if (redoStack.length > 0) {
        var newById = redoStack[0];
        var newUndoStack = [byId].concat(undoStack);
        var newRedoStack = redoStack.slice(1);
        var undoRedoVersion = _this.props.value.undoRedoVersion;

        _this.props.onChange(tslib_1.__assign({}, _this.props.value, {
          undoRedoVersion: undoRedoVersion + 1,
          byId: newById,
          undoStack: newUndoStack,
          redoStack: newRedoStack
        }));
      }
    };

    return _this;
  }

  BlockEditorControl.prototype.render = function () {
    var _this = this;

    var _a = this.props,
        UiComponent = _a.UiComponent,
        onChange = _a.onChange,
        value = _a.value,
        controlUiProps = _a.controlUiProps;
    return React.createElement(UiComponent, tslib_1.__assign({
      value: value,
      onChange: onChange,
      addRow: this.addRow,
      addCol: this.addCol,
      addImage: this.addImage,
      addLayer: this.addLayer,
      addMarkDown: this.addMarkDown,
      moveInDirection: this.moveInDirection,
      copyFocused: this.copyFocused,
      paste: this.paste,
      focusNode: function (nodeId, focus) {
        var node = value.byId[nodeId];
        node && _this.focusNode(node, focus);
      },
      removeFocused: this.removeFocused,
      updateBlock: this.updateBlock,
      undo: this.undo,
      redo: this.redo
    }, controlUiProps));
  };

  BlockEditorControl.defaultProps = {
    UiComponent: BlockEditorControlDefaultUI
  };
  return BlockEditorControl;
}(React.Component);

exports.AbsoluteLayerBlock = AbsoluteLayerBlock;
exports.BlockBreadCrumbs = BlockBreadCrumbs;
exports.BlockEditor = BlockEditor;
exports.BlockEditorControl = BlockEditorControl;
exports.BlockEditorControlDefaultUI = BlockEditorControlDefaultUI;
exports.DraggableColBlock = DraggableColBlock;
exports.DraggableRowBlock = DraggableRowBlock;
exports.ImageBlock = ImageBlock;
exports.MarkdownBlock = MarkdownBlock;
exports.Preview = Preview;
exports.ResizableBlock = ResizableBlock;
exports.addCol = addCol;
exports.addImage = addImage;
exports.addLayer = addLayer;
exports.addMarkDown = addMarkDown;
exports.addRow = addRow;
exports.create = create;
exports.deepCopy = deepCopy;
exports.defaultRenderEditBlock = defaultRenderEditBlock;
exports.defaultRenderPreviewBlock = defaultRenderPreviewBlock;
exports.destroy = destroy;
exports.focusNode = focusNode;
exports.getDragPositionRelativeToTarget = getDragPositionRelativeToTarget;
exports.getDragPositionRelativeToTarget2 = getDragPositionRelativeToTarget2;
exports.hasDescendent = hasDescendent;
exports.move = move;
exports.moveInDirection = moveInDirection;
exports.newBlockId = newBlockId;
exports.onDragStart = onDragStart;
exports.onDropped = onDropped;
exports.onDropped2 = onDropped2;
exports.parseTypes = parseTypes;
exports.paste = paste;
exports.pasteChild = pasteChild;
exports.placeNodeInParent = placeNodeInParent;
exports.reinsertIntoList = reinsertIntoList;
exports.removeNode = removeNode;
exports.update = update;
exports.updateNode = updateNode;
//# sourceMappingURL=react-movable-block-editor.cjs.development.js.map
